//@+ CheckArithOverflow = no

/*@ axiomatic Divides {

predicate divides(integer x, integer y) ;

axiom divides_zero:
  \forall integer q;
  divides(q, 0) ;

axiom divides_self:
  \forall integer q;
  divides(q, q) ;

axiom divides_modulo:
  \forall integer q, integer x, integer y;
  divides(q, y) && y != 0 ==>
  (divides(q, x) <==> divides (q, x % y)) ;

} */

class GCD {

/*@ requires  a >= 0 && b >= 0 ;
    decreases b ;
    ensures
      \result >= 0 &&
      \forall integer q;
      divides(q, a) && divides(q, b) <==>
      divides(q, \result) ; */
static int gcd (int a, int b)
{
  if (b == 0)
    return a;
  else
    return gcd (b, a % b);
}

/*@ requires  a >= 0 && b >= 0 ;
    ensures
      \result >= 0 &&
      \forall integer q;
      divides(q, a) && divides(q, b) <==>
      divides(q, \result) ; */
static int gcd_iter (int a, int b)
{
  /*@ loop_invariant
	a >= 0 && b >= 0 &&
	\forall integer q;
	divides(q, a) && divides(q,  b) <==>
	divides(q, \at(a, Pre)) && divides(q, \at(b, Pre)) ;
      loop_variant b ; */
  while (b > 0) {
    int r = a % b;
    a = b;
    b = r;
  }
  return a;
}

}
